$.widget("wildix.chat", {
    _create: function () {
        this._helpButton = this.element.find(".chat-help-button");
        this._extensionInput = this.element.find(".chat-extension-input");
        this._extensionInput.on("keypress", $.proxy(this._onExtensionInputKeyPress, this));
        this._extensionAddButton = this.element.find(".chat-add-button");
        this._extensionAddButton.on("click", $.proxy(this._onExtensionAddButtonPressed, this));
        this._infoButton = this.element.find(".chat-info-button");
        this._infoButton.on("click", $.proxy(this._onInfoButtonClick, this));

        this._usersListEl = this.element.find(".chat-users-list");
        this._usersListEmptyEl = this.element.find(".chat-users-empty");
        this._bodyHeaderEl = this.element.find(".chat-body-header");
        this._bodyMessagesEl = this.element.find(".chat-body-messages");
        this._bodyInputEl = this.element.find(".chat-body-input");
        this._bodyInputEl.on("keypress", $.proxy(this._onBodyInputElKeyPress, this));

        this._chats = {};
        this._active_chat = null;

        $.template("chatUserItem", '<li jid="${jid}"><a href="#">${name}</a></li>');
        $.template("chatMessagesItem", '<div jid="${jid}" class="chat-body-messages-container"></div>');
        $.template("chatMessageItem", '<div class="chat-body-message-body">${body}</div>');
        $.template("chatMessageHeaderItem", '<div class="chat-body-message-header">${name}</div>');
    },

    // --------------------
    // Public API
    // --------------------

    showChat: function(user) {
        var jid = user.getJid();
        if (!this._chats[jid]) {
            this._chats[jid] = this._createChat(user);
        }

        var chat = this._chats[jid];

        if (this._active_chat && this._active_chat !== chat) {
            this._active_chat.tabEl.removeClass("active");
            this._active_chat.messagesEl.addClass("hidden");
        }

        this._usersListEmptyEl.addClass("hidden");
        this._bodyHeaderEl.html(user.getJid());
        this._bodyMessagesEl.removeClass("disabled")
        this._bodyInputEl.attr("disabled", false);

        this._active_chat = chat;
        this._active_chat.tabEl.addClass("active");
        this._active_chat.messagesEl.removeClass("hidden");
    },

    showIncomingMessage: function(user, message) {
        var jid = user.getJid();
        var chat = this._chats[jid];

        if (!chat) {
            return;
        }

        var messagesEl = chat.messagesEl;
        var splitter = chat.splitter;
        if (splitter != "incoming") {
            chat.splitter = "incoming";
            $.tmpl("chatMessageHeaderItem", {name: user.getJid()}).appendTo(messagesEl);
        }

        $.tmpl("chatMessageItem", {body: this._escape(message.getBody())}).appendTo(messagesEl);
    },

    showOutgoingMessage: function(user, message) {
        var jid = user.getJid();
        var chat = this._chats[jid];

        if (!chat) {
            return;
        }

        var messagesEl = chat.messagesEl;
        var splitter = chat.splitter;

        if (splitter != "outgoing") {
            chat.splitter = "outgoing";
            $.tmpl("chatMessageHeaderItem", {name: "Me"}).appendTo(messagesEl);
        }

        $.tmpl("chatMessageItem", {body: this._escape(message.getBody())}).appendTo(messagesEl);
    },

    disconnected: function() {
        this._extensionInput.attr("disabled", true);
        this._extensionAddButton.addClass("disabled");
        this._bodyInputEl.attr("disabled", true);
    },

    reconnected: function() {
        this._extensionInput.attr("disabled", false);
        this._extensionAddButton.removeClass("disabled");

        if (this._active_chat) {
            this._bodyInputEl.attr("disabled", false);
        }
    },

    // --------------------
    // Private functionality
    // --------------------

    _createChat: function(user) {
        var tabEl = $.tmpl("chatUserItem", {jid: user.getJid(), name: user.getJid()}).appendTo(this._usersListEl);
        tabEl.on("click", $.proxy(this._onTabPressed, this));
        var messagesEl = $.tmpl("chatMessagesItem", {jid: user.getJid()}).appendTo(this._bodyMessagesEl);

        return {
            user: user,
            splitter: "",
            tabEl: tabEl,
            messagesEl: messagesEl
        }
    },

    _closeChat: function(user) {

    },

    _onTabPressed: function(event) {
        var jid = $(event.target).parent("li").attr("jid");
        var chat = this._chats[jid];

        if (chat) {
            var user = chat.user;
            this.element.trigger("chat", [user.getJid()]);
        }
    },

    /**
     * Fires when user type something in message input.
     *
     * @param event
     * @private
     */
    _onBodyInputElKeyPress: function(event) {
        if (event.keyCode == 13) {
            var chat = this._active_chat;
            var user = chat.user;
            var body = $.trim($(this._bodyInputEl).val())

            if (body.length == 0) {
                return false;
            }

            this.element.trigger("send", [user, body]);
            this._bodyInputEl.val("");
            return false;
        }

        return true;
    },

    _onExtensionInputKeyPress: function(event) {
        if (event.keyCode == 13) {
            this._onExtensionAddButtonPressed();
            return false;
        }

        return true;
    },

    /**
     * Fires when user are pressed to "add" chat button.
     *
     * @private
     */
    _onExtensionAddButtonPressed: function() {
        var extension = this._extensionInput.val();
        if (extension && extension != '') {
            this.element.trigger("chat", [extension+'']);
        }

        this._extensionInput.val("");
    },

    _onInfoButtonClick: function() {
        $("#chat-help-modal").modal("show");
    },

    _escape: function(str) {
        return str.replace(/&/g,'&amp;').replace(/</g,'&lt;').replace(/>/g,'&gt;') ;
    }
});