$.widget("wildix.dialer", {
    _create: function () {
        this._activeDevices = this.options.activeDevices;
        this._selectedDevice = this._getPreferredDevice(this._activeDevices, null);

        this.element.removeClass("hidden");
        this._inputEl = $(this.element).find("#po-dialer-input");
        this._callButtonEl = $(this.element).find("#po-dialer-call-button");
        this._callButtonNonEmptyEl = $(this.element).find("#po-dialer-non-empty-button-view");
        this._callButtonEmptyEl = $(this.element).find("#po-dialer-empty-button-view");
        this._selectionsButtonEl = $(this.element).find("#po-dialer-selections-button");
        this._selectedDescriptionEl = $(this.element).find("#po-device-description");
        this._selectionExampleEl = $(this.element).find("#po-dialer-device-selection-template");
        this._selectionsEl = $(this.element).find("#po-dialer-device-selections");

        this._refreshDevices(this._activeDevices);
        this._refreshSelectedDevice(this._selectedDevice);
        this._bind();
    },

    _bind: function() {
        this._callButtonEl.on("click", $.proxy(this._onCallButtonClick, this));
    },

    _onCallButtonClick: function() {
        if (this._callButtonEl.hasClass("disabled")) {
            return;
        }

        var to = this._inputEl.val();
        var numberRegExp = new RegExp(/^((\+\d+)|(#|\*|\d)+)$/);
        var phoneRegExp = new RegExp(/^(\+\d{10,15})*$/);

        if(!to.match(numberRegExp) && !to.match(phoneRegExp)) {
            return;
        }

        this.element.trigger("dial", [this._selectedDevice, to]);
    },

    _refreshDevices: function(devices) {
        this._selectionsEl.find("li").remove();

        for (var i=0; i < devices.length; i++) {
            var device = devices[i];
            var element = this._selectionExampleEl.clone();
            element.find("a").html(device.getName());
            element.removeAttr("id");
            element.removeClass("hidden");
            element.addClass("po-device-selection");
            element.attr("data-id", device.getId());
            element.on("click", $.proxy(this._onDeviceChanged, this));

            if (device === this._selectedDevice) {
                element.addClass("po-device-selection-active");
            }

            this._selectionsEl.append(element);
        }

        if (devices.length == 0) {
        	this._inputEl.attr("disabled", true);
            this._callButtonEl.addClass("disabled");
            this._selectionsButtonEl.addClass("disabled");
            this._callButtonNonEmptyEl.hide();
            this._callButtonEmptyEl.show();
        } else {
        	this._inputEl.attr("disabled", null);
            this._callButtonEl.removeClass("disabled");
            this._selectionsButtonEl.removeClass("disabled");
            this._callButtonNonEmptyEl.show();
            this._callButtonEmptyEl.hide();
        }
    },

    _refreshSelectedDevice: function(device) {
        if (device) {
            this._selectedDescriptionEl.html(device.getName());
        } else {
            this._selectedDescriptionEl.html("-");
        }
    },

    _onDeviceChanged: function(event) {
        var target = $(event.target).parent("li");
        var id = target.attr("data-id");
        var device = null;

        for (var i=0; i < this._activeDevices.length; i++) {
            if (id == this._activeDevices[i].getId()) {
                device = this._activeDevices[i];
                break;
            }
        }

        if (device) {
            this._selectedDevice = device;
            this._refreshDevices(this._activeDevices);
            this._refreshSelectedDevice(device);
        }
    },

    enable: function() {
    	if(this._activeDevices.length > 0){
    		this._inputEl.attr("disabled", null);
            this._callButtonEl.removeClass("disabled");
            this._selectedDescriptionEl.removeClass("disabled");
    	}
    },

    disable: function() {
        this._inputEl.attr("disabled", true);
        this._callButtonEl.addClass("disabled");
        this._selectedDescriptionEl.addClass("disabled");
    },

    onDevicesChanged: function (devices) {
        this._activeDevices = devices;
        this._selectedDevice = this._getPreferredDevice(devices, this._selectedDevice);

        this._refreshDevices(this._activeDevices);
        this._refreshSelectedDevice(this._selectedDevice);
    },

    _getPreferredDevice: function(devices, selectedDevice) {
        var exist = false;
        var preferred = null;
        if (selectedDevice) {
            for (var i=0; i < devices.length; i++) {
                if (devices[i].getId() === selectedDevice.getId()) {
                    preferred = devices[i];
                    exist = true;
                }
            }
        }

        if (!exist) {
            preferred = devices[0] || null;
        }

        return preferred;
    }
});