$.widget("wildix.porow", {
    _create: function () {
        this._call = this.options.call;
        this._activeDevices = this.options.activeDevices;
        this._selectedDevice = this._getPreferredDevice(this._activeDevices, null);
        this._activeCalls = this.options.activeCalls;
        this._otherCalls = this._filterOtherCalls(this._activeCalls, this._call);

        this.element.removeClass("hidden");

        this._channelColumn = $(this.element).find(".channel-column");
        this._statusColumn = $(this.element).find(".status-column");
        this._calleeColumn = $(this.element).find(".callee-column");
        this._durationColumn = $(this.element).find(".duration-column");

        this._actionsEl = $(this.element).find(".callee-actions");
        this._hangupActionEl = $(this._actionsEl).find(".hangup-button");
        this._holdActionEl = $(this._actionsEl).find(".hold-button");
        this._resumeActionEl = $(this._actionsEl).find(".resume-button");
        this._transferActionEl = $(this._actionsEl).find(".transfer-button");
        this._transferPopOverEl = $(this._actionsEl).find(".transfer-popover");
        this._transferBlindInputEl = $(this._actionsEl).find(".transfer-blind-input");
        this._transferBlindInputEl.on("keypress", $.proxy(this._onTransferBlindInputKeyPressed, this));
        this._transferBlindSubmitEl = $(this._actionsEl).find(".transfer-blind-submit-button");
        this._transferBlindSubmitEl.on("click", $.proxy(this._onTransferBlindSubmitPressed, this));
        this._transferAttendantContainerEl = $(this._actionsEl).find(".transfer-attendant-container");
        this._transferAttendantTitleEl = $(this._actionsEl).find(".transfer-attendant-title");
        this._startRecordActionEl = $(this._actionsEl).find(".start_record-button");
        this._stopRecordActionEl = $(this._actionsEl).find(".stop_record-button");
        this._answerActionEl = $(this._actionsEl).find(".answer-button");
        this._answerSelectionEl = $(this._answerActionEl).find(".answer-using-button");
        this._answerDescriptionEl = $(this._answerActionEl).find(".answer-description");
        this._answerSelectionsEl = $(this._answerActionEl).find(".answer-selection-button");
        this._declineActionEl = $(this._actionsEl).find(".decline-button");

        this._refresh();
        this._bind();

        //

        $.template("consoleAttendantTransferLink", '<div><a href="#" channel="${channel}">${channel}</a></div>');
    },

    _bind: function() {
        // Bind actions
        this._hangupActionEl.on("click", $.proxy(this._onHangupPressed, this));
        this._transferActionEl.on("click", $.proxy(this._onTransferPressed, this));
        this._holdActionEl.on("click", $.proxy(this._onHoldPressed, this));
        this._resumeActionEl.on("click", $.proxy(this._onResumePressed, this));
        this._startRecordActionEl.on("click", $.proxy(this._onStartRecordPressed, this));
        this._stopRecordActionEl.on("click", $.proxy(this._onStopRecordPressed, this));
        this._answerSelectionEl.on("click", $.proxy(this._onAnswerPressed, this));
        this._declineActionEl.on("click", $.proxy(this._onDeclinePressed, this));
    },

    _onAnswerPressed: function() {
        this.element.trigger("answer", [this._call, this._selectedDevice]);
    },

    _onHangupPressed: function() {
        this.element.trigger("hangup", [this._call]);
    },

    _onTransferPressed: function() {
        if (this._transferActionEl.hasClass("active")) {
            this._collapseTransferPopOver();
        } else {
            this._expandTransferPopOver();
        }
    },

    _expandTransferPopOver: function() {
        // Hide other popovers
        $(".transfer-button").removeClass("active");
        $(".transfer-popover").hide();

        this._transferPopOverEl.show();
        this._transferActionEl.addClass("active");
    },

    _collapseTransferPopOver: function() {
        this._transferPopOverEl.hide();
        this._transferActionEl.removeClass("active");
    },

    _onTransferBlindInputKeyPressed: function(event) {
        if (event.keyCode == 13) {
            this._onTransferBlindSubmitPressed();
            return false;
        }

        return true;
    },

    _onTransferBlindSubmitPressed: function() {
        var to = this._transferBlindInputEl.val();
        if (to.length > 0) {
            this._transferBlindInputEl.val("");
            this._collapseTransferPopOver();
            this.element.trigger("blind_transfer", [this._call, to]);
        }
    },

    _onTransferAttendantPressed: function(event) {
        var channel = $(event.target).attr("channel");
        var to = null;

        for (var i=0; i < this._otherCalls.length; i++) {
            var otherCall = this._otherCalls[i];
            if (otherCall.getChannel() == channel) {
                to = otherCall;
                break;
            }
        }

        if (to) {
            this.element.trigger("attendant_transfer", [this._call, to]);
        }

        this._collapseTransferPopOver();
    },

    _onDeclinePressed: function() {
        this.element.trigger("decline", [this._call]);
    },

    _onHoldPressed: function() {
        this.element.trigger("hold", [this._call]);
    },

    _onResumePressed: function() {
        this.element.trigger("resume", [this._call]);
    },

    _onStartRecordPressed: function() {
        this.element.trigger("start_record", [this._call]);
    },

    _onStopRecordPressed: function() {
        this.element.trigger("stop_record", [this._call]);
    },

    _onAnswerDeviceChanged: function(event) {
        var target = $(event.target).parent("li");
        var id = target.attr("data-id");
        var device = null;

        for (var i=0; i < this._activeDevices.length; i++) {
            if (id == this._activeDevices[i].getId()) {
                device = this._activeDevices[i];
                break;
            }
        }

        if (device) {
            this._selectedDevice = device;
            this._refreshAnswerSelectedDevice(device);
        }
    },

    _refresh: function() {
        this._channelColumn.find("div").html(this._call.getChannel());
        this._statusColumn.html(this._call.getFormattedState());
        this._calleeColumn.find(".callee-name").html(this._call.getFormattedCalleeName());

        this._refreshDuration();
        this._refreshActions();
    },

    _refreshActions: function() {
        var actions = this._getSupportedActionsList();

        switch (this._call.getState()) {
            case "ring":
                if (this._call.isIncoming()) {
                    actions.answer = true;
                    actions.decline = true;
                } else {
                    actions.hangup = true;
                }
                break;
            default:
                if (this._call.isOnHold()) {
                    actions.resume = true;
                } else {
                    actions.hold = true;
                }

                actions.hangup = true;
                actions.transfer = true;

                if (this._call.isRecorded()) {
                    actions.stop_record = true;
                } else {
                    actions.start_record = true;
                }
                break;
        }

        var ael = $(this.element).find(".callee-actions");
        for (var key in actions) {
            var button = $(ael).find("." + key + "-button");
            if (actions[key]) {
                button.removeClass("hidden");
            } else {
                button.addClass("hidden");
            }

            // Special condition for answer
            if (key === "answer") {
                this._refreshAnswerButton(button);
                this._refreshAnswerSelectedDevice(this._selectedDevice);
            }

            // Special condition for transfer
            if (key === "transfer") {
                this._refreshTransferButton();
            }
        }
    },

    _refreshAnswerButton: function(button) {
        $(button).find('.dropdown-menu > li').remove();

        var container = $(button).find('.dropdown-menu');
        var section = $(button).find(".answer-selection-item");
        var matches = 0;
        for (var i=0; i < this._activeDevices.length; i++) {
            var device = this._activeDevices[i];
            var el = section.clone();

            if (device.constructor.name === "AnyDevice" ||
                device.constructor.name === "MobilityDevice") {
                continue;
            }

            el.attr("data-id", device.getId());
            el.find("a").html(device.getName());
            el.removeClass("hidden");
            el.on("click", $.proxy(this._onAnswerDeviceChanged, this));
            container.append(el);
            matches++;
        }


        if (matches == 0) {
            $(button).find(".answer-disabled-button").removeClass("hidden");
            $(button).find(".answer-using-button").addClass("hidden");
            $(button).find(".answer-selection-button").addClass("disabled");
        } else {
            $(button).find(".answer-disabled-button").addClass("hidden");
            $(button).find(".answer-using-button").removeClass("hidden");
            $(button).find(".answer-selection-button").removeClass("disabled");
        }
    },

    _refreshTransferButton: function() {
        this._transferAttendantContainerEl.empty();

        if (this._otherCalls.length > 0) {
            this._transferAttendantTitleEl.show();

            for (var i=0; i < this._otherCalls.length; i++) {
                var call = this._otherCalls[i];
                var channel = call.getChannel();
                var el = $.tmpl("consoleAttendantTransferLink", {channel: channel}).appendTo(this._transferAttendantContainerEl);
                el.on("click", $.proxy(this._onTransferAttendantPressed, this));
            }
        } else {
            this._transferAttendantTitleEl.hide();
        }
    },

    _refreshAnswerSelectedDevice: function(selectedDevice) {
        if (selectedDevice) {
            this._answerDescriptionEl.html(selectedDevice.getName());
        } else {
            this._answerDescriptionEl.html("-");
        }
    },

    _refreshDuration: function() {
        if (this._call.isRinging()) {
            this._durationColumn.html("-");
        } else {
            this._durationColumn.html(this._call.getFormattedDuration());
        }
    },

    _getSupportedActionsList: function() {
        return {
            hangup: false,
            transfer: false,
            hold: false,
            resume: false,
            start_record: false,
            stop_record: false,
            answer: false,
            decline: false
        };
    },

    _getPreferredDevice: function(devices, selectedDevice) {
        var preferred = null;
        if (selectedDevice) {
            for (var i=0; i < devices.length; i++) {
                var device = devices[i];
                if (device.constructor.name === "AnyDevice" ||
                    device.constructor.name === "MobilityDevice") {
                    continue;
                }
                if (device.getId() === selectedDevice.getId()) {
                    preferred = device;
                    exist = true;
                }
            }
        }

        if (preferred === null) {
            preferred = devices[0] || null;
        }

        return preferred;
    },

    enable: function() {
        this._actionsEl.removeClass("disabled");
        this._hangupActionEl.removeClass("disabled");
        this._holdActionEl.removeClass("disabled");
        this._resumeActionEl.removeClass("disabled");
        this._transferActionEl.removeClass("disabled");
        this._startRecordActionEl.removeClass("disabled");
        this._stopRecordActionEl.removeClass("disabled");
        this._answerActionEl.removeClass("disabled");
        this._answerSelectionEl.removeClass("disabled");
        this._answerSelectionsEl.removeClass("disabled");
        this._declineActionEl.removeClass("disabled");
    },

    disable: function() {
        this._actionsEl.addClass("disabled");
        this._hangupActionEl.addClass("disabled");
        this._holdActionEl.addClass("disabled");
        this._resumeActionEl.addClass("disabled");
        this._transferActionEl.addClass("disabled");
        this._transferActionEl.addClass("disabled");


        this._startRecordActionEl.addClass("disabled");
        this._stopRecordActionEl.addClass("disabled");
        this._answerSelectionEl.addClass("disabled");
        this._answerSelectionsEl.addClass("disabled");
        this._declineActionEl.addClass("disabled");


    },

    onCallsChanged: function(calls) {
        this._activeCalls = calls;
        this._otherCalls = this._filterOtherCalls(this._activeCalls, this._call);
        this._refreshTransferButton();
    },

    onDevicesChanged: function (devices) {
        this._devices = devices;
    },

    onDurationChanged: function() {
        this._refreshDuration();
    },

    onCallUpdated: function() {
        this._refresh();
    },

    _filterOtherCalls: function(calls, call) {
        var result = [];
        for (var i=0; i < calls.length; i++) {
            var otherCall = calls[i];
            if (otherCall.getChannel() != call.getChannel()) {
                result.push(otherCall);
            }
        }

        return result;
    }
});