
var statuses = {
	'1': 'Unallocated (unassigned) number',
	'16': 'Normal call clearing',
	'17': 'User busy',
	'18': 'No user responding',
	'19': 'No answer',
	'21': 'Call rejected',
	'200': 'FAX detected'
}

var options = {
	pbxUrl: '',
	applicationId: '',
	redirectUri: '',
	campaignID: '4',
	dialplanCG: 'dialcg',
	queueID: '2',
	reserv: 1,
	uriNumbers: ''
}

var numbers = [];
var wtapi = null;
var queuesView = null;
var numbersView = null;

function oauth2Callback(params) {
	wtapi.handleCallbackData(params);
}

function isHasChanges(options) {
	for (const [optionName, optionValue] of Object.entries(options)) {
		if (localStorage.getItem(optionName) !== optionValue) {
			return true;
		}
	}
	return false;
}

$(document).ready(function() {
	$.ajaxSetup({
		crossDomain: true,
		xhrFields: {
			withCredentials: false
		}
	});

    numbersView = $('#numbers-view');
    numbersView.on('click', '.actions .start', onStartPredictiveDialer);
    numbersView.on('click', '.actions .pause', onPausePredictiveDialer);
    numbersView.on('click', '.actions .resume', onResumePredictiveDialer);


    var settingsModal = $("#settings-modal");

    $('#settings-btn').on('click', function(){
    	settingsModal.modal('show');
    });

    $('#settings-modal-save-button').on('click', function(){

    	let pbxUrl = $('#settings-modal-host').val();
		let re = new RegExp("^(http|https)://", 'i');
		if (pbxUrl && !re.test(pbxUrl)) {
			pbxUrl = 'https://' + pbxUrl;
		}

		const applicationId = $('#settings-modal-application').val();
		const redirectUri = $('#settings-modal-redirect-uri').val();
		const predictive_campaignID = $('#settings-modal-campaignID').val();
		const predictive_dialplanCG = $('#settings-modal-dialplanCG').val();
		const predictive_queueID = $('#settings-modal-queueID').val();
		const predictive_reserv = $('#settings-modal-reserv').val();
		const predictive_uriNumbers = $('#settings-modal-uriNumbers').val();

		const hasChanges = isHasChanges({
			pbxUrl,
			applicationId,
			redirectUri,
			predictive_campaignID,
			predictive_dialplanCG,
			predictive_queueID,
			predictive_reserv,
			predictive_uriNumbers
		});

		if (hasChanges === true) {
			localStorage.setItem("host", pbxUrl);
			localStorage.setItem("applicationId", applicationId);
			localStorage.setItem("redirectUri", redirectUri);
			localStorage.setItem("predictive_campaignID", predictive_campaignID);
			localStorage.setItem("predictive_dialplanCG", predictive_dialplanCG);
			localStorage.setItem("predictive_queueID", predictive_queueID);
			localStorage.setItem("predictive_reserv", predictive_reserv);
			localStorage.setItem("predictive_uriNumbers", predictive_uriNumbers);

			if (wtapi) {
				wtapi.deleteSavedCredentials();
				wtapi.disconnect();
			}

			initialise();
		}
		settingsModal.modal('hide');
    });

    initialise();

});

$.template("numbersTemplate", "" +
		"<table cellpadding='0' cellspacing='0'>" +
		"<thead>" +
		"<tr>" +
			"<th>ID</th>" +
			"<th>Name</th>" +
			"<th>Number</th>" +
			"<th>Status</th>" +
		"</tr>" +
		"</thead>" +
		"<tbody>" +
			"{{each(id, name, number, status) numbers}}" +
			"<tr id='number_${id}'>" +
				"<td class='id'>${id}</td>" +
				"<td class='name'>${name}</td>" +
				"<td class='number'>${number}</td>" +
				"<td class='status'>{{if statuses[status]}}${statuses[status]}{{else}}${status}{{/if}}</td>" +
			"</tr>" +
			"{{/each}}" +
		"</tbody>" +
		"</table>");

function calcRedirectUri() {
	let redirectUri = '';
	if (typeof(localStorage.redirectUri) === "string" && localStorage.redirectUri !== '') {
		redirectUri = localStorage.redirectUri;
	} else {
		let loc = window.location;
		let pathname = loc.pathname
			.replace('index.html', '')
			.replace('predictiveDialer/', '');
		redirectUri = loc.origin + pathname + 'oauth2redirect.html';
	}

	return redirectUri;
}

function initialise(){

	options = {
		pbxUrl: localStorage.getItem("host"),
		applicationId: localStorage.getItem("applicationId"),
		redirectUri: calcRedirectUri(),
		campaignID: localStorage.getItem("predictive_campaignID"),
		dialplanCG: localStorage.getItem("predictive_dialplanCG"),
		queueID: localStorage.getItem("predictive_queueID"),
		reserv: localStorage.getItem("predictive_reserv"),
		uriNumbers: localStorage.getItem("predictive_uriNumbers")
	}

	$('#settings-modal-host').val(options.pbxUrl);
	$('#settings-modal-application').val(options.applicationId);
	$('#settings-modal-redirect-uri').val(options.redirectUri);
	$('#settings-modal-campaignID').val(options.campaignID);
	$('#settings-modal-dialplanCG').val(options.dialplanCG);
	$('#settings-modal-queueID').val(options.queueID);
	$('#settings-modal-reserv').val(options.reserv);
	$('#settings-modal-uriNumbers').val(options.uriNumbers);

	if(wtapi && wtapi.connected){
		wtapi = null;
		$('#queues-view .queues-wrapper').html('');

		numbers = [];
		changeState('stoped');

		queuesView.queues('destroy');
		queuesView = null;

		$('#numbers').html('');

		if(processInterval){
			clearInterval(processInterval);
			processInterval = null;
		}
		if(updateNumbersInterval){
			clearInterval(updateNumbersInterval);
			updateNumbersInterval = null;
		}
	}

	if(!options.pbxUrl || !options.applicationId || !options.redirectUri) {
		return false;
	}

	wtapi = new WTAPI();
	wtapi.setOptions(options);

	wtapi.on("connected", onConnected);
	wtapi.on("disconnected", onDisconnected);

	wtapi.on("restoreCredentials", onRestoreCredentials);
	wtapi.restoreSavedCredentials();
}

function initQueue(needConnect = true) {
	queuesView = $('#queues-view');
	queuesView.queues();

	queuesView.on("member_add", onQueueMemberAdd);
	queuesView.on("member_pause", onQueueMemberPause);
	queuesView.on("member_resume", onQueueMemberResume);
	queuesView.on("member_remove", onQueueMemberRemove);

	// Queues demo
	wtapi.queues.on("queue_added", onQueueAdded);
	wtapi.queues.on("queue_removed", onQueueRemoved);
	wtapi.queues.on("member_added", onQueueMemberAdded);
	wtapi.queues.on("member_updated", onQueueMemberUpdated);
	wtapi.queues.on("member_removed", onQueueMemberRemoved);
	wtapi.queues.on("call_added", onQueueCallAdded);
	wtapi.queues.on("call_removed", onQueueCallRemoved);
	wtapi.queues.on("user_join", onQueueUserJoin);
	wtapi.queues.on("user_left", onQueueUserLeft);

	if (needConnect) {
		wtapi.connect();
	}
}

function onRestoreCredentials(data) {
	console.log('Restored credentials', data);
	initQueue(data === null);
}

var _state = 'stoped';
function changeState(state){
	if(_state!=state){
		switch (state) {
			case 'stoped': numbersView.find('.actions .start').attr('disabled', 'disabled');
							numbersView.find('.actions .pause').attr('disabled', 'disabled');
							numbersView.find('.actions .resume').attr('disabled', 'disabled');
							numbersView.find('.actions .start').show();
							numbersView.find('.actions .pause').hide();
							numbersView.find('.actions .resume').hide();
				break;
			case 'ended': numbersView.find('.actions .start').removeAttr('disabled');
							numbersView.find('.actions .pause').attr('disabled', 'disabled');
							numbersView.find('.actions .resume').attr('disabled', 'disabled');
							numbersView.find('.actions .start').show();
							numbersView.find('.actions .pause').hide();
							numbersView.find('.actions .resume').hide();

				break;
			case 'ready': numbersView.find('.actions .start').removeAttr('disabled');
							numbersView.find('.actions .resume').hide();
							numbersView.find('.actions .pause').hide();
				break;
			case 'not_ready': numbersView.find('.actions .start').attr('disabled', 'disabled');
								numbersView.find('.actions .resume').hide();
								numbersView.find('.actions .pause').hide();
				break;
			case 'inProgress': numbersView.find('.actions .start').attr('disabled', 'disabled');
								numbersView.find('.actions .pause').removeAttr('disabled', 'disabled');
								numbersView.find('.actions .resume').attr('disabled', 'disabled');
								numbersView.find('.actions .start').hide();
								numbersView.find('.actions .pause').show();
								numbersView.find('.actions .resume').show();
				break;
			case 'autoPaused':
			case 'paused': numbersView.find('.actions .start').attr('disabled', 'disabled');
							numbersView.find('.actions .resume').removeAttr('disabled', 'disabled');
							numbersView.find('.actions .pause').attr('disabled', 'disabled');
				break;
			default: numbersView.find('.actions .resume').hide();
					numbersView.find('.actions .pause').hide();
				break;
		}

		_state = state;
	}
}


function getState(){
	return _state;
}

var _queue = null;
function getQueue(){
	if(_queue === null){
		_queue = wtapi.queues.getQueue(options.queueID);
	}
	return _queue;
}



var updateNumbersInterval = null;
function maybeReady(){
	if(numbers.length>0){
		var queue = getQueue();
		if(queue){
			changeState('ready');
			updateNumbersInterval = setInterval(updateNumbers, 10000);
		}else{
			changeState('not_ready')
		}
	}else{
		changeState('not_ready')
	}
}

function getNextNumber(){
	var number;
	for(var i=0; i < numbers.length; i++){
		if(!numbers[i].called){
			number = numbers[i];
			break;
		}
	}
	return number;
}

function getCountMembers(){

	var count = 0;
	var queue = getQueue()
	if(queue){
		var members = queue.getMembers();
		for(var i=0; i < members.length; i++){
			if(!members[i].isPaused()){
				count++;
			}
		}
		if(count > 0){
			count += options.reserv*1;
		}
	}
	return count;
}

function updateNumbers(){
	var params = {};
	if(numbers && numbers.length>0){
		params.update = true;
	}
	$.ajax({
		type: "GET",
		url: options.uriNumbers,
		dataType: 'json',
		data: params,
		timeout: 3000,
		success: function(data, textStatus, jqXHR){
			if(!numbers || numbers.length == 0){
				numbers = data;
				maybeReady();
				$('#numbers').html($.tmpl("numbersTemplate", {'numbers': numbers, 'statuses': statuses}));

			}else if(data.length > 0){
				$.each(data, function(i,el){
					if(statuses[el.status]){
						$('#numbers #number_'+el.id+' .status').text(statuses[el.status]);
					}else{
						$('#numbers #number_'+el.id+' .status').text(el.status);
					}
				})
			}
		}
	});
}

function checkCallFiles(){
	//console.log('checkCallFiles');
	$.ajax({
		type: "GET",
		url: options.pbxUrl + '/api/v1/PredictiveDialer/calls',
		dataType: 'json',
		timeout: 3000,
		success: function(data, textStatus, jqXHR){
			var countMembers = getCountMembers() - data.result.length;
			if(countMembers > 0){
				for(; countMembers > 0; countMembers--){
					makeCall();
				}
			}
		}
	});
}

function makeCall(){
	var number = getNextNumber();
	if(number){
		number.called = true;
		$.ajax({
			type: "POST",
			url: options.pbxUrl + '/api/v1/PredictiveDialer/calls',
			dataType: 'json',
			timeout: 2000,
			data: {
				numberid: number.id,
				number: number.number,
				dialplanCG: options.dialplanCG,
				campaignID: options.campaignID
			},
			success: function(data, textStatus, jqXHR){
				console.log(number, data.result)
			}
		});
	}else{
		if(processInterval){
			clearInterval(processInterval);
			processInterval = null;
			changeState('ended');

			numbers = [];
			changeState('stoped');
			if(processInterval){
				clearInterval(processInterval);
				processInterval = null;
			}
			if(updateNumbersInterval){
				clearInterval(updateNumbersInterval);
				updateNumbersInterval = null;
			}

		}
	}
}

function checkMembers(){
	var countMembers = getCountMembers();
	if(countMembers > 0){
		$('#queues-error').hide();
		if(getState() == 'autoPaused'){
			onResumePredictiveDialer();
		}
	}else{
		$('#queues-error').show();
		if(getState() == 'inProgress'){
			onPausePredictiveDialer();
			changeState('autoPaused');
		}
	}
}


var processInterval = null;
function onStartPredictiveDialer(){
	changeState('inProgress');
	var countMembers = getCountMembers();
	if(countMembers > 0){
		processInterval = setInterval(checkCallFiles, 5000);
		for(; countMembers > 0; countMembers--){
			makeCall();
		}
	}
	checkMembers();
}

function onPausePredictiveDialer(){
	changeState('paused');
	if(processInterval){
		clearInterval(processInterval);
		processInterval = null;
	}
	if(updateNumbersInterval){
		clearInterval(updateNumbersInterval);
		updateNumbersInterval = null;
	}
}

function onResumePredictiveDialer(){
	var countMembers = getCountMembers();
	if(countMembers > 0){
		changeState('inProgress');
		processInterval = setInterval(checkCallFiles, 5000);
		updateNumbersInterval = setInterval(updateNumbers, 10000);
		checkCallFiles();
		updateNumbers();
	}
}

function onConnected(){
	wtapi.queues.subscribe(onQueuesSubscribed);

	$.ajaxSetup({
		headers: {
			'Authorization': 'Bearer ' + wtapi.getCredentials().accessToken
		}
	});

	updateNumbers();
}

function onDisconnected(){

}


//----------------------------------------
// Start Queues events
//----------------------------------------

function onQueueAdded(queue) {
	if(queue.getId() == options.queueID){
		queuesView.queues('addQueue', queue);
		checkMembers();
	}
}

function onQueueRemoved(queue) {
	if(queue.getId() == options.queueID){
		queuesView.queues('removeQueue', queue);
		checkMembers();
	}
}

function onQueueMemberAdded(queue, member) {
	if(queue.getId() == options.queueID){
		queuesView.queues('addMember', queue, member);
		checkMembers();
	}
}

function onQueueMemberUpdated(queue, member) {
	if(queue.getId() == options.queueID){
		queuesView.queues('updateMember', queue, member);
		checkMembers();
	}
}

function onQueueMemberRemoved(queue, member) {
	if(queue.getId() == options.queueID){
		queuesView.queues('removeMember', queue, member);
		checkMembers();
	}
}

function onQueueCallAdded(queue, member, call) {
	if(queue.getId() == options.queueID){
		queuesView.queues('updateMemberCalls', queue, member);
	}
}

function onQueueCallRemoved(queue, member, call) {
	if(queue.getId() == options.queueID){
		queuesView.queues('updateMemberCalls', queue, member);
	}
}

function onQueueUserJoin(queue, user) {
	if(queue.getId() == options.queueID){
		queuesView.queues('addUser', queue, user);
	}
}

function onQueueUserLeft(queue, user) {
	if(queue.getId() == options.queueID){
		queuesView.queues('removeUser', queue, user);
	}
}


function onQueuesSubscribed() {
	maybeReady();
	setInterval(onQueueUpdateDuration, 1000);
}

function onQueueUpdateDuration() {
	if(!wtapi){
		return;
	}
    var queues = wtapi.queues.getQueues();
    for (var i=0; i < queues.length; i++) {
    	if(queues[i].getId() != options.queueID){
    		continue;
    	}
        var queue = queues[i];
        var members = queue.getMembers();
        var users = queue.getUsers();

        for (var c=0; c < members.length; c++) {
            queuesView.queues('updateMemberTime', queue, members[c]);
        }

        for (var c=0; c < users.length; c++) {
            queuesView.queues('updateUserDuration', queue, users[c]);
        }
    }
}

function onQueueMemberAdd(event, queue, extension) {
    var callback = function(error) {
        if (error) {
            onError(error);
        }

        queuesView.queues('enable');
    };

    queuesView.queues('disable');
    wtapi.queues.add(queue, extension, callback);
}

function onQueueMemberPause(event, queue, member) {
    var callback = function(error) {

        queuesView.queues('enable');
    };

    queuesView.queues('disable');
    wtapi.queues.pause(queue, member, callback);
}

function onQueueMemberResume(event, queue, member) {
    var callback = function(error) {

        queuesView.queues('enable');
    };

    queuesView.queues('disable');
    wtapi.queues.resume(queue, member, callback);
}

function onQueueMemberRemove(event, queue, member) {
    var callback = function(error) {

        queuesView.queues('enable');
    };
    queuesView.queues('disable');
    wtapi.queues.remove(queue, member, callback);
}

//----------------------------------------
//End Queues events
//----------------------------------------

