/**
 * Settings window for queues demo.
 * Note:
 * There are four options for demo extension, password, url, queues which are takes
 * from global window context (window.extension, window.password ...), if such variables
 * not present in window, settings based on localStorage will used
 */

var settingsModal = null;

let options = {
    host: localStorage.getItem("host") || '',
    applicationId: localStorage.getItem("applicationId") || '',
    redirectUri: calcRedirectUri(),
    queues: JSON.parse(localStorage.getItem("wallboard_queues") || '[]')
}

function isHasChanges(options) {
    for (const [optionName, optionValue] of Object.entries(options)) {
        if (localStorage.getItem(optionName) !== optionValue) {
            return true;
        }
    }
    return false;
}

function calcRedirectUri() {
    let redirectUri = '';
    if (typeof(localStorage.redirectUri) === "string" && localStorage.redirectUri !== '') {
        redirectUri = localStorage.redirectUri;
    } else {
        let loc = window.location;
        let pathname = loc.pathname
            .replace('index.html', '')
            .replace('wallboard/', '');
        redirectUri = loc.origin + pathname + 'oauth2redirect.html';
    }

    return redirectUri;
}

if (window.applicationId == undefined || window.redirectUri == undefined || window.host == undefined){

    for (var option in options){
        if (options.hasOwnProperty(option)){
            window[option] = options[option];
        }
    }

    $(document).ready(function() {

        $(settingsHtml).appendTo($("#settings").show()).hide();
        settingsModal = $("#settings-modal");

        if (!options.applicationId || !options.redirectUri || !options.host){
            $('#status').hide();
            showSettings();
        }
        $('#settings-btn').on('click', function(){
            showSettings();
        });
        $('#settings-modal-save-button').on('click', onClickSettingsSave);
    });
}

function showSettings(){
    settingsModal.modal('show');

    $('#settings-modal-application').val(options.applicationId);
    $('#settings-modal-redirect-uri').val(options.redirectUri);
    $('#settings-modal-host').val(options.host);
    for (var i = 0; i < 4; i++){
        if (options.queues[i]){
            $('#settings-modal-callgroup-'+(i+1)).val(options.queues[i].id);
            $('#settings-modal-sla-wait-'+(i+1)).val(options.queues[i].sla_wait);
            $('#settings-modal-sla-percentage-'+(i+1)).val(options.queues[i].sla_percentage);
        }
    }
}

function onClickSettingsSave(){
    let host = $('#settings-modal-host').val();
    let re = new RegExp("^(http|https)://", 'i');
    if (host && !re.test(host)) {
        host = 'https://' + host;
    }

    options = {
        applicationId: $('#settings-modal-application').val(),
        redirectUri: $('#settings-modal-redirect-uri').val(),
        host: host,
        queues: []
    };
    for (var i = 0; i < 4; i++){
        options.queues.push({
            id:$('#settings-modal-callgroup-'+(i+1)).val(),
            sla_wait:$('#settings-modal-sla-wait-'+(i+1)).val(),
            sla_percentage:$('#settings-modal-sla-percentage-'+(i+1)).val(),
        });
    }

    const queues = JSON.stringify(options.queues);

    const hasChanges = isHasChanges({
        host: options.host,
        applicationId: options.applicationId,
        redirectUri: options.redirectUri,
        wallboard_queues: queues
    });

    if (hasChanges === true) {
        localStorage.setItem("host", options.host);
        localStorage.setItem("applicationId", options.applicationId);
        localStorage.setItem("redirectUri", options.redirectUri);
        localStorage.setItem("wallboard_queues", queues);

        if (wtapi) {
            wtapi.disconnect();
            wtapi.deleteSavedCredentials();
        }

        document.location.reload();
    }
    settingsModal.modal('hide');
}


var settingsHtml =
'<div id="settings-modal" class="modal fade" tabindex="-1" role="dialog" aria-hidden="true">\n\
     <div class="modal-dialog">\n\
         <div class="modal-content">\n\
             <div class="modal-header">\n\
                 <button type="button" class="close" data-dismiss="modal" aria-hidden="true">×</button>\n\
                 <h4 class="modal-title">Settings</h4>\n\
             </div>\n\
             <div class="modal-body">\n\
                 <table>\n\
                     <tr>\n\
                         <td width="180px">PBX host</td>\n\
                         <td><input id="settings-modal-host" class="form-control" type="text"></td>\n\
                     </tr>\n\
                     <tr>\n\
                         <td>OAuth 2.0 Application ID</td>\n\
                         <td><input id="settings-modal-application" class="form-control" type="text"></td>\n\
                     </tr>\n\
                     <tr>\n\
                         <td>Redirect URI</td>\n\
                         <td><input id="settings-modal-redirect-uri" class="form-control" disabled type="text"></td>\n\
                     </tr>\n\
                     <tr>\
                         <td colspan="2">\
                             <table class="settings-modal-queues" style="margin-top: 10px;">\
                                 <tr>\
                                     <td>Callgroup Id</td><td>SLA wait</td><td>SLA percentage</td>\
                                 </tr>\
                                 <tr>\
                                     <td><input id="settings-modal-callgroup-1" class="form-control" type="text" ></td>\
                                     <td><input id="settings-modal-sla-wait-1" class="form-control" type="text"></td>\
                                     <td><input id="settings-modal-sla-percentage-1" class="form-control" type="text"></td>\
                                 </tr>\
                                 <tr>\
                                     <td><input id="settings-modal-callgroup-2" class="form-control" type="text" ></td>\
                                     <td><input id="settings-modal-sla-wait-2" class="form-control" type="text"></td>\
                                     <td><input id="settings-modal-sla-percentage-2" class="form-control" type="text"></td>\
                                 </tr>\
                                 <tr>\
                                     <td><input id="settings-modal-callgroup-3" class="form-control" type="text" ></td>\
                                     <td><input id="settings-modal-sla-wait-3" class="form-control" type="text"></td>\
                                     <td><input id="settings-modal-sla-percentage-3" class="form-control" type="text"></td>\
                                 </tr>\
                                 <tr>\
                                     <td><input id="settings-modal-callgroup-4" class="form-control" type="text" ></td>\
                                     <td><input id="settings-modal-sla-wait-4" class="form-control" type="text"></td>\
                                     <td><input id="settings-modal-sla-percentage-4" class="form-control" type="text"></td>\
                                 </tr>\
                             </table>\n\
                         </td>\n\
                     </tr>\n\
                 </table>\n\
             </div>\n\
             <div class="modal-footer">\n\
                 <button id="settings-modal-save-button" type="button" class="btn btn-primary fa fa-check">Ok</button>\n\
             </div>\n\
         </div>\n\
     </div>\n\
 </div>\n\
';
